#############################################################################
#
# Keep our Bugzilla types/subtypes in one place :-)
#
# Author:  Chris Weyl (cpan:RSRCHBOY), <cweyl@alumni.drew.edu>
# Company: No company, personal work
# Created: 01/04/2009 05:29:50 AM PST
#
# Copyright (c) 2009 Chris Weyl <cweyl@alumni.drew.edu>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
#############################################################################

package Fedora::Bugzilla::Types;

use strict;
use warnings;

use MooseX::Types::Moose     qw{ Str Object ArrayRef HashRef };
use MooseX::Types::DateTimeX 'DateTime';

use DateTime::Format::Bugzilla;
use Email::Address;

use namespace::clean;

use MooseX::Types 
    -declare => [ 'Str20', 'BugzillaDateTime', 'EmailAddress' ];

our $VERSION = 0.04;

subtype Str20
    => as Str
    => where { length $_ <= 20 }
    ;

subtype BugzillaDateTime
    => as DateTime
    => where { $_->formatter->isa('DateTime::Format::Bugzilla') }
    ;

coerce BugzillaDateTime
    => from DateTime
    => via { $_->set_formatter(DateTime::Format::Bugzilla->new()); $_ }
    ;

=begin
coerce BugzillaDateTime
    => from Str
    => via { 
        my $dt = DateTime::Format::Bugzilla->parse_datetime($_);
        $dt->set_formatter(DateTime::Format::Bugzilla->new());
        return $dt;
        }
    ;
=cut

class_type 'Email::Address';

subtype EmailAddress()
    => as 'Email::Address'
    ;

for my $type (EmailAddress, 'Email::Address') {
    
    coerce $type
        => from Str
        => via { my @a = Email::Address->parse($_); pop @a }
        => from ArrayRef
        => via { Email::Address->new(@$_) }
        => from HashRef
        => via { Email::Address->new(%$_) }
        ;
}

1;

__END__

=head1 NAME

Fedora::Bugzilla::Types - Moose types and coercions for Fedora::Bugzilla

=head1 SYNOPSIS

	use <Module::Name>;
	# Brief but working code example(s) here showing the most common usage(s)

	# This section will be as far as many users bother reading
	# so make it as educational and exemplary as possible.


=head1 DESCRIPTION

A full description of the module and its features.
May include numerous subsections (i.e. =head2, =head3, etc.)


=head1 SUBROUTINES/METHODS

A separate section listing the public components of the module's interface.
These normally consist of either subroutines that may be exported, or methods
that may be called on objects belonging to the classes that the module provides.
Name the section accordingly.

In an object-oriented module, this section should begin with a sentence of the
form "An object of this class represents...", to give the reader a high-level
context to help them understand the methods that are subsequently described.


=head1 DIAGNOSTICS

A list of every error and warning message that the module can generate
(even the ones that will "never happen"), with a full explanation of each
problem, one or more likely causes, and any suggested remedies.


=head1 CONFIGURATION AND ENVIRONMENT

A full explanation of any configuration system(s) used by the module,
including the names and locations of any configuration files, and the
meaning of any environment variables or properties that can be set. These
descriptions must also include details of any configuration language used.


=head1 SEE ALSO

L<Fedora::Bugzilla>, L<MooseX::Types>, L<Moose::Util::TypeConstraints>.

=head1 AUTHOR

Chris Weyl  <cweyl@alumni.drew.edu>


=head1 LICENSE AND COPYRIGHT

Copyright (c) 2009 Chris Weyl <cweyl@alumni.drew.edu>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the 

    Free Software Foundation, Inc.
    59 Temple Place, Suite 330
    Boston, MA  02111-1307  USA

=cut

